﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Windows;
using System.Windows.Controls;

/*
 *         Progetto: Localizzatore di utenti mediante datagrammi UDP
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 25/10/2015
 *  
 *      Descrizione: Modulo servent - Applicazione di rete P2P per inviare il nome dell'utente 
 *                   in broadcast mediante UDP.
 *                   Riceve i nomi degli utenti connessi alla rete con un metodo asincrono.
 *                   Mostra l'elenco degli utenti e lo aggiorna sulla base delle informazioni ricevute.  
 */

namespace LocalizzatoreUDP
{
    // Classe utilizzata per organizzare le informazioni nella ListView.

    public class Info
    {
        public string IndirizzoIP { get; set; }
        public string NomeUtente { get; set; }
    }

    /// <summary>
    /// Logica di interazione per MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        const int portaUdpRicezione = 4100;      // Porta UDP dedicata alla ricezione dei messaggi
        const string indirizzoBroadcast = "10.0.255.255";

        const char separatore = '|';    // Separa i campi del messaggio inviato/ricevuto 

        bool faseModificaNome = true;   // Se il flag è "vero", il nome utente non è ancora valido (modifica del nome in corso...)

        public MainWindow()
        {
            InitializeComponent();
        }

        // Inserisce oppure aggiorna un utente nella ListView 

        private void AggiungiNuovaInfo(string indirizzo, string utente)
        {
            bool trovato = false;

            foreach (Info i in elenco.Items)
            {
                if (i.IndirizzoIP.Equals(indirizzo))
                {
                    // Individuata la riga da aggiornare
                    i.NomeUtente = utente;
                    trovato = true;
                    break;
                }
            }

            if (!trovato)
                elenco.Items.Add(new Info() { IndirizzoIP = indirizzo, NomeUtente = utente });


            elenco.Items.Refresh();     // Mostra gli aggiornamenti effettuati sullo schermo
        }

        // Rimuove un utente dalla ListView

        private void RimuoviInfo(string indirizzo)
        {

            foreach (Info i in elenco.Items)
            {
                if (i.IndirizzoIP.Equals(indirizzo))
                {
                    elenco.Items.Remove(i);
                    break;
                }
            }

            elenco.Items.Refresh();
        }

        private void ElaboraDatiRicevuti(IPEndPoint hostRemoto, string messaggio) {

            // Scompone il messaggio nelle parti necessarie al completamento dell'elaborazione

            string[] parti = messaggio.Split(new char[] { separatore });
            string azione = parti.Length > 0 ? parti[0].ToUpper() : "";
            string utenteRemoto = parti.Length > 1 ? parti[1] : "";

            if (azione.Equals("NUOVO"))
            {
                // Occorre aggiungere il nuovo utente remoto alla ListView e inviargli un messaggio di 
                // risposta contenente il nome utente locale

                AggiungiNuovaInfo(hostRemoto.Address.ToString(), utenteRemoto);

                string utenteLocale = (faseModificaNome == false ? tbUtente.Text : "<Non disponibile>");
                InviaMessaggio(hostRemoto.Address, "RISPOSTA" + separatore + utenteLocale);

            }
            else if (azione.Equals("RISPOSTA"))
            {
                // La risposta fornita dall'host remoto permette di aggiornare l'elenco degli utenti connessi
                AggiungiNuovaInfo(hostRemoto.Address.ToString(), utenteRemoto);
            }
            else if (azione.Equals("CHIUDI"))
            {
                // Occorre eliminare dalla ListView l'utente che ha segnalato la chiusura del collegamento.
                RimuoviInfo(hostRemoto.Address.ToString());
            }

        }

        private void InviaMessaggio(IPAddress destinatario, string messaggio)
        {
            using (UdpClient mittente = new UdpClient())
            {
                byte[] buffer = Encoding.UTF8.GetBytes(messaggio);
                mittente.Send(buffer, buffer.Length, new IPEndPoint(destinatario, portaUdpRicezione));
            }
        }


        // Metodo asincrono per ricevere ed elaborare i messaggi provenienti dagli altri servent

        private async void Ascolta()
        {
            using (UdpClient client = new UdpClient(portaUdpRicezione)) {

                while (true)
                {
                    try
                    {
                        UdpReceiveResult risultato = await client.ReceiveAsync();
                        ElaboraDatiRicevuti(risultato.RemoteEndPoint, Encoding.UTF8.GetString(risultato.Buffer));

                    }
                    catch (Exception) {}
                }

            }

        }

        private void ControllaBottone()
        {
            if (btnRileva != null && tbUtente != null)
            {
                string utente = tbUtente.Text.Trim();
                btnRileva.IsEnabled = (utente.Length > 0);
            }
        }

        private void tbUtente_TextChanged(object sender, TextChangedEventArgs e)
        {
            faseModificaNome = true;
            ControllaBottone();        
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Ascolta();
        }

        private void btnRileva_Click(object sender, RoutedEventArgs e)
        {
            faseModificaNome = false;
            InviaMessaggio(IPAddress.Parse(indirizzoBroadcast), "NUOVO" + separatore + tbUtente.Text);
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            InviaMessaggio(IPAddress.Parse(indirizzoBroadcast), "CHIUDI");
        }
    }
}
